--=============================================================================
--! @file serdesCalibrateFSM_rtl.vhd
--=============================================================================
--
-------------------------------------------------------------------------------
-- --
-- UoB , USC
-- --
------------------------------------------------------------------------------- --
--
--! @brief Finite-state machine to control calibration and reset signals to
--! Iserdes, IDelay
--! based on code by Alvaro Dosil\n
--
--! @author  Alvaro Dosil (David Cussans , David.Cussans@bristol.ac.uk)
--
--! @date 22/Feb/2014
--
--! @version v0.1
--
--! @details
--
-------------------------------------------------------------------------------
--! \n\n<b>Last changes:</b>\n
-------------------------------------------------------------------------------
--! @todo Implement a periodic calibration sequence\n
--! <another thing to do> \n
-
  LIBRARY ieee;
USE ieee.std_logic_1164.all;

entity serdesCalibrateFSM is
  port (
    serdes_reset_i : in std_logic;      --! Take high to start calibration
    fsm_clock_i : in std_logic;
    busy_idelay_m_i ,  busy_idelay_m_i : in std_logic;     --! Status of IDELAY elements.
    calibrate_o , reset_o : out std_logic  --! Output from FSM to calibrate and reset lines
    );
end entity serdesCalibrateFSM;

architecture rtl of serdesCalibrateFSM is

  signal s_calibration  : std_logic := '0';         -- Start calibration
  --! Calibration FSM state values
  type state_values is (st0, st1, st2, st3, st4);
  signal pres_state, next_state: state_values := st0;
  signal s_initial_cal  : std_logic := '1';         -- Start up calibration flag
  signal s_rst_FSM      : std_logic := '0';         -- IODELAY reset
  
begin  -- rtl

  --! Calibration start condition
  s_calibration <= serdes_reset_i or s_initial_cal;

  --! Calibration FSM register
  statereg: process(fabricClk_i, serdes_reset_i)
  begin
    if rising_edge(fabricClk_i) then
      pres_state <= next_state;     -- Move to next state
      
    end if;
  end process statereg;


  --! Calibration FSM combinational block
  fsm: process(pres_state, s_calibration, s_busy_idelay_m)
  begin
    next_state <= pres_state;
    -- Default values
    s_Rst_FSM <= '0';
    s_cal_FSM <= '0';
    
    case pres_state is
      
      -- st0 - IDLE
      when st0=>
        if ( s_calibration = '1') then 
          next_state <= st1;            -- Next state is "st1 - SEND CALIBRATION SIGNAL"
        end if;
        
        -- st1 - SEND CALIBRATION SIGNAL
      when st1=>
        s_cal_FSM <= '1';
        s_initial_cal <= '0';
        if s_busy_idelay_m = '1' then 
          next_state <= st2;            -- Next state is "st2 - WAIT BUSY = '0'"
        end if;
        
        -- st2 - WAIT BUSY = '0'
      when st2=>
        if s_busy_idelay_m = '0' then 
          next_state <= st3;            -- Next state is "st3 - RESET STATE"
        end if;
        
        -- st3 - RESET STATE
      when st3=>
        s_Rst_FSM <= '1';
        if s_busy_idelay_m = '1' then 
          next_state <= st4;              -- Next state is "st4 - WAIT BUSY = '0'"
        end if;
        
        -- st4 - WAIT BUSY = '0'
      when st4=>
        if s_busy_idelay_m = '0' then 
          next_state <= st0;              -- Next state is "st0 - IDLE"
        end if;
    end case;
    
  end process fsm;

  calibrate_o <= s_cal_FSM;
  reset_o <= s_Rst_FSM;
                
end rtl;
